# MIT License
#
# Copyright (c) 2018 Ross Wilson
# Copyright (c) 2024, Santiago Peñate Vera
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
"""
Source of the 'error' and 'pending' tiles data.

The code here is generated from the:
    error_tile.png
    pending_tile.png
files in the pyslipqt/examples/graphics directory using:
    /usr/local/bin/img2py <file>

The captured output is cleaned up and saved in this file as data.
"""

import base64


# the 'pending' tile data
def getPendingImage():
    """Generate 'pending' image from embedded data."""

    return base64.b64decode(
        "iVBORw0KGgoAAAANSUhEUgAAAQAAAAEACAIAAADTED8xAAAACXBIWXMAAAsT"
        "AAALEwEAmpwYAAAAB3RJTUUH4gIJAxggK6MKCwAAIABJREFUeNrsvWlwXNeV"
        "JnjOufdtmUgsiY0ACJIAuK8StVGWbJVU8tJVtmZctqpcsstR1TUTMRUVPdM/"
        "JmY6JmI6OmJ+TfT8618z86NnpvYql2W7ynZJlmXtCzdREkESJEGCWAgQ+5bL"
        "e+/ec+bHSyQBrkmKbovg+yIlIm9mvnzv5fnuPefcs+DLL78MKVI8qKD0FqRI"
        "CZAiRUqAFClSAqRIkRIgRYqUAClSpARIkSIlQIoUKQFSpEgJkCJFSoAUKVIC"
        "pEiREiBFipQAKVKkBEiRIiVAihQpAVKkSAmQIkVKgBQpUgKkSJESIEWKlAAp"
        "UqQESJEiJUCKFCkBUqRICZAiRUqAFClSAqRIkRIgRYqUAClSpARIkSIlQIoU"
        "KQFSpEgJkCJFSoAUKVICpEiREiBFipQAKVKkBEiRIiVAihQpAVKkSAmQIkVK"
        "gBQpUgKkSJESIEWKlAApUqQESJEiJUCKFCkBUqRICZAiRUqAFClSAqRICZAi"
        "RUqAFClSAqRIkRIgRYqUAClSpARIkSIlQIoUKQFSpEgJkCJFSoAUKVICpEiR"
        "EiBFipQAKVKkBEiRIiVAihT3I3R6C2rB8vT08tJkuDSzVAhLYRhFRsSCgFak"
        "HTcIvFxDkMnk65ra/Fw+vV0pAdYDwuW5qaGBy9OzczNzU/NLc4sLS4tLYVSI"
        "CssmjsRYACBHa8f1MlkvyOXqcvlcfWtTrqU139re3rWpB/zG9DamBLjPYIyZ"
        "vnj8/IXLI6NXRq6Mz89cDpcKhESoEZGQiNDVfuXOCUDM4dxSeWZ+DuGijQEh"
        "aGxoynds6tywuauju6enbfNOolTV/JwCX3755fQuVFCeHzz98ZlzQ6cHL82M"
        "XzShUeQopSjnou86WU/5LvkkHoIG1CAAGCPEArHYguUoipcjLoVciERsbGIn"
        "G3R09+3Y0r1vZ0/vtp1lJ9WOUgJ8LhHH8djJIx+dOv3pmYGlqXFNmrSLdY6X"
        "r3MafW4D2yzShKY5Ludi9tn4pvpZVVK6qINFV80TzSBOA01JNF+M54u8FMU2"
        "RiVN7Zv379x64JGHO3v2ImJ6w1MCfI4wNXLuyAfvH/v09OLUZU0aPcfNZ/y2"
        "et4CZiOUu0tR1rpAzERAifiuFmJBAQYRYWBQ1ojoRZ0Z8vUI4jCXphbMfFmi"
        "0JK0dGw59PDuh598KtfUlUpeSoDfPKIoOnf87dfePXx56CxZQN9zm7Pepnqz"
        "TcIdYbkhdkKHiJQG6wIqJEcIEREBEZTEKCioGEQEGMQKRMAhihUr1mgTzLru"
        "GVefk+LIAs+VTFQm3+3dvverX3xs4+5DqWGQEuA3ieLs5bdf/9UHJ44vT0+7"
        "rk+NXmZL3uyHYl8onriiRJPyATwBDQJAgsAAAKwEDSAiKwEAEhSU1cuCWLEh"
        "qjJwxBFaXULvjKs+luLoLC9H1oa5lq5nn3ri8ae+5GSbUhH8zUJ95zvfeQAv"
        "e/byhX98+Z+OHf8gXi66uUywMa+fypafisM245IiT0EWtY/gCApiTGgQAAQE"
        "AMgQMYEAAqAhYCQmZAIDYBGZUEhpRA/JJW0VCpY6Yt5EgV9HBYIYivOzQ5dH"
        "lhcWNnW1O0EulcKUAP9Fcfnimb//wcsXzn6ihFRjkNnZar5Epa2Rg9pxNeZQ"
        "uQSMwiiMyhIKoiAwVASdAQFREC2iIDFVniYjjIAgIiCAhBQgatJGMUK8MXZa"
        "Am85I5ExS+Wx8bHJqfkt3Z1+tj4VxJQA/6Wkf/DE3/7Dj8YuDTjKU/mM/0Te"
        "PM62ybrgSB1KBkSQDAgAsSATMwgDCAIjMSIgMFgAYBRBEkRGISFDyChKQKDK"
        "EGAAC0iIHoImXdRhQ4wd5NmsXbZYMpNTY+NXZvu62/xcqgulBPj1Y/zCwN/9"
        "44/HL51znMBpybhfagj3MiChT5BB1KgNogAIgCAxCKAGJEAEEUHFaBGEEQGF"
        "BQRYQBiASQREQNkKQxAwUZNYEEWQUSmEACjUTAKd4DoZnjNQlpmZy+NTC9u2"
        "bvSCdB1ICfBr1ftHLvz9D18euzjguIHbllHP1Ee97BhH16FkgACEQTEioxUg"
        "BhZUDMAADMyoAIABDGpAFnCYFKMCBCEFAIyKgQWFERiFwQKDEAqKgGIEA4hI"
        "PhAglSluYTeb4akYS3Z2dmJurtDbu8X1M6lEpgT4taA0N/7yj/9p8Mwnjvac"
        "loz6Yr3ZJK7RVI82ABQEiwhIFoCBGJFRAIEAmIghmccJEAQRQASBkQHAIlok"
        "BrKICGCSDwoiakZiAEE0KICMK1qTD6KIihDn2Q8ydsZI0UzNTUgU9vVuRsdP"
        "hTIlwD1GGIZvvvLTYyc+VEyqKXC+0GA3g8uaciQeggVlVKKxCKJ1BVwRV8Bh"
        "BCCLYkgbIqOEQSGIIQAQFkQkEUFAEUSwFjSi1cweo8viiWgRABTEGNGiTgxr"
        "AXBBo8ISxE3iSYbnQi7E47PT9b7XuWXb3V3jm7/8p6nJ0ShcQFLWguu6qXDX"
        "ggciGO78iXc++Ogol43TEPgHmso97MREOQREHQIAMoqtN+yyOx7QCMqcQARA"
        "QDmENgq3FiPNzqKLMYpFUoKGRBCtAAMqAUAEkIa4pKRhMMfjIvMMFkEDNSB2"
        "4PL2RQCgeRcsakBrQQJRRtl5ifeyv9wcnpsuzS/88p0Puza2d2x95C6ukdlM"
        "Tc4sl2hoeD4ZqaurC4IgCIJczvP9esdxXNdVSqVC/2ARYHr0/K/ePVKcnXXq"
        "sv6W5nibaEPkOICCMTJA1Bj5BUe97ZVOh9PzV5aWClEhtLEgopPRmbpMY2Nj"
        "XW8dPgrhhrKe8RAFAEAQEQQRIzDNoSpq/5dB6VT58uzY0uJyWIiZWWlyAydb"
        "X9fY2Oj3+vg4hBvKPOe4VolmCCAou+UwNAfAWWiwocxODP3yzSPfbN2SbWi+"
        "08tEUH42BwCImOwxh2EYhuHi4uLk5NXwDa0pCDzXdR0pOFqhk/M8x3U0ub4D"
        "ShNYnVVKPTi71OucAHEcH37/vZGLA9rx3bY6+xCIEo0aHcSYxLHgiH/EWz66"
        "ODJ0eWls6fojzMH8GFwO+oPOUx0tj7eYh62ySBGBBraoCOL6yPsoWH5n+dL5"
        "c0vjNzjCNMxeguHMyUznxx2tD7XIk2KCWC85rAEDUJFjspE85OplHybN6TMf"
        "bz268dBvf/1Or9Sray6Xo+vHRQQRE1Yk/4ahLS9PxFFIAKCmCIEIBFAhasWg"
        "CC2SIiImIqWUQnR85Vgrkqwepm3rl1IC3B8YHTh8/OQptIgNHu0ITJ0Qk3IV"
        "GLLZWC2p+G179tiZxRXRr6+v37JlS76lJQgCY8zi/Pzw8PD4+HhprjR49MKV"
        "0cmd4zv0F7XJGBUqQrGO1a+75944N3NhNjlCU1NTT09PPp93PC8Ow7m5ueHh"
        "4ampqeJ08fz04OWhy9subM9+ORtuKTrTvjjieEoWlW217pZ6Lkbl+eUPP/q0"
        "d9v2tk3b7+hKibRIWEuoaRwu2LhEQIBAAqiQSAgQkISUAlEKCQEdhYIAAAI2"
        "ioEQOQYkim26AtwfMMX5Y8dPL0yOu5lM0NEQdwvGqB0NAOJYmqTCa4UzR8+a"
        "ogGAvr6+np6e7du3d3d3t6wQYGFhYWRk5Ny5c+fPn+/v71+eWP7k1U/2lvd6"
        "z7niCHuMb+On/3KyOF0EgK6url27du3evXvTpk35fN513TAM5+fnR0ZGLly4"
        "MDQ8/MmJE8WZ0qdvfbJ5YfOGf9Ue7wy9hAOOY8s23iHeZD0XzPjwuU+PHXt2"
        "49Zfhx4SRws2KpGiJK4pxXomwOCpj/oH+h3l6EbfbCUgISEiEhSYh8VXF899"
        "OGhjW1dXt3v37ueff/6hhx5yHGf1ERoaGjZt2vTUU0+dPHnyxz/+8ZkzZ5aX"
        "l0++dXK/vz96Mfbf9E794lQi/bt27fr617/+2GOPrT5CNpvN5/O9vb3PPPPM"
        "qVOn2lpaTpw4MT09ffH4kDWmi7qiLZG76CpHqVBFgdGbXJx1ZC76eGBw18VT"
        "nX177+0NsdGCicsKUVgQ0rSEde0GjaLovbfePH+6X2cDv7vJ7gSMUDkKNQJD"
        "+fXwzAdnbWRbWlqefPLJP/7jP+7r67uFh6StrW3fvn1j4+OzMzPlQrlULrVL"
        "65VjV6bOTwPAnj17/uzP/mzPnj23OEJra+v+/fuVUtPT04uLiwtXFoNyUNeV"
        "E1+QlbBI0XIjOHOaF6Olxel8Q+OW7Xtqv96xsbE4jnEFVau3agCYeNGEJSQk"
        "RMCr/yEhISAgrPxNyUtq5XVAJCFEFABEZK5r7UkJ8HnH2OkPX3/3iCmVdEuA"
        "e3wOhICUUuijPcpn3huIl+PGxsann376u9/9bi53+5DMIAh27dw5ODg4MTER"
        "LUfzo/PL08umbPbt2/fd7363t7f39qut1jt37lRKXb58eWlpaWFyoVW10HYi"
        "gyDAsVgtuqzNfMSFstFB7+b2TF3+nhDAhIs2DpEIEVICrLGd1uvSdv7C8MLU"
        "CDqO25S1rYIxIiIQmEkePTlani0DwP79+1988cUgCGo8Zj6ff/755zdu3MiG"
        "l68USvNlrfX+/ft37txZ+4k9//zzTz31VENDg43tuf5BGRB2GRWSJgwl3she"
        "U6Adf3T4/PC5s5/9PiBiFC3HYSFVeR4gAsxdGbkwOgkWMOtIXif2Hgqih+Z4"
        "NH1+GgAOHjz4wgsv1NXV3dGRH3nkkb5t21Y/feihh+5szVXq61//esKZxdHF"
        "xeOLGCMKEhEycQao3kdflRYWLo2Ox3Fcq35vb6x9mbhoowJgmn32IBHgyvC5"
        "scuXlHLcep83AJYBgYDAztmpiWkbsdZ69+7dfX19d3pkz/P6enq0rjgPmltb"
        "t27deqcHqa+vf/rppzdu3AgAE+NXzDkrKISICjEUbgNV53quNzQyMTN8usZj"
        "BvoXgs51Pp9SVF5M5/4HjgATV6aXpyfRc6jO5aygAWQADfa0XbiykEz/+/fv"
        "v7uDNzQ0tLS0JHN5S/4uK508+uijyUqyOLYUno+AIEmpkQhsk6g6l0jPzoxO"
        "jF+u1ehfPOzKB7zKrReXCyZeFEnF/wEjgFmcuDwxCwLga8gptgIAIgIA5dli"
        "caoEAC1tbdu23WXYWV1jY319PQBks9na7Ydr4Pv+1t5e13VtbAuLixIKAJBC"
        "tMiOYOCAr0sLS5NXZu/A8bX8nqc+FHESzcfEi7+W+7u+POfrcB9gZnZ8cm5W"
        "KUdnNNcDxklmFnIoy4VSInwd7e23Psj8/Pzhw4eXlpZ6tm3bt2vXau++S5S4"
        "O5VSVV2oisHBwZMnTyqlDhw40N3dfYuvaG1t3bBhw/Dw8HKhkJ9tpnpKJiQ2"
        "AFmFnoNLODE7X1y8nKnvrHE2ixfe9RqgFB4w0RLpVO9/IAkwPTE7Nz+lSJGv"
        "bU4gqWGlAJckLkeJM6ep6VYpiOPj43//gx+c/OST2dnZbdu2zX35y88++2x1"
        "XzYJMgOAcrlcLpdXf/D4J5+88tOfnjhxgoguXrz4la98ZceOHTf7lpaWlpaW"
        "luHh4bAYyYxIHUCMAAAxcoMon4j0zNzc3MSVWgmQaD5L78YcCe5KN3sfUALM"
        "Ly6XF2a18snVxhMqAipEES6jMQYAMplMJnOr3Ks333zznbfeSt48MDAQBEF3"
        "d/f27ZXgnIWFhYWFBQAolUoJExIUCoVXf/azw4cPJ09ff/11EdmyZYvneTf8"
        "luppmMhISUQEE1cVM/uIviaixcXZxeX5Oy2jFdCREEDMbtQ2FfEHzgYoLBVM"
        "ZMAn0IQMYjFJaxS+OiMmJsGNP14oAEAi/QmGhoYuXbpUfTo1NTUzM1NxN01N"
        "Vd85ODiYfLaKUqk0MjJS00kziBVkEEEwII6gJlSqtDi3MFe4i5vgwRGSU2LT"
        "6P8HjAAYLhTKJQRErcBBiBFFJBEvDYkaUywWi8XiTafPIJibm1s90tjYWN0u"
        "mJ2dTab/ChmuXDlz5kzVO3TNroKfyTQ0NNzsi6qnQZrQQbAiVlAEAAQFHYWu"
        "CoulsFy4u1vhuUcU9v86FnlmTgnwOcXSUlwox0gKCFkDJC4gK2IAXNCeSoR4"
        "fn7+pneE6Kmnnjp48GDyNJfL7d27d9++fcnTkydPVpUcAPj0009Pn6646ru7"
        "u/fs2dO+Yl5v3rz5wL59ra2tN/uiqamp6elpAPACR+oZDXCyUolAjEk9dgAo"
        "l81d3w1fH1VyjM095YC51fqZ2gC/YRgul+MIEVEhKAQjYgURgYED8QMvMV5H"
        "R0ettTeLXdu/f7/WetOWLXEcd7S3P/7448nUvry8/MHhwzMzM6TJz7umzNFi"
        "dPbs2cHBwb6+PiJ65plncrnc4OAgIu7Zs+fWm8Sjo6Ojo6MA4Gd8zCHHjFbE"
        "ClgUEiAQQkIK49AYc727qaapWiBwPy4bFHkM4N7YA5a91Aj+/MLGoTVWEQIi"
        "oIAVZAIUAcYyZuozQatXmgpHR0f7+/tvsRe2e/fu3bt3XzP4i1/84sypUwCQ"
        "afN792+5PDQxvTh77Nixxnz+X//xHwdBkMvlnnnmmWeeeea25zk5OTkxMcHM"
        "ylPZXJZF0FSK7AIDMQgBIBBRbITiZdB32WyGGXx9IrLA9nH1mW3isJDzGv/f"
        "V078+9955LVUBfo8gplZGACQoKJNJzYAg5RBdTjZ1iwA9Pf3v/Pee1EU1X7k"
        "995774033pibm0OFrZubcZdq62xxc46InDh+/Gc/+5m1dyBe77///rvvvgsA"
        "ua6s35mBMrABsFipMs1X1QxrrOXPqnL4dMLRR6x1PstBSsVc44a/JBhKjeDP"
        "8QrAwpYBQBgABQGQKw82DI7kNzQ6dRoAPv7oo1deeaV2ef3hD384PDwMAPlt"
        "DfVbGnEZ3B6vbUcrEExPT7/++us/+clPVntFb4Fjx46988475XIZFbZsyFML"
        "SARgRThxAyXFGKXqsPrM8g+M4NFHvvrAyl0qMFEx07Thb0rRIADAlVQF+twS"
        "epV/InH+iMBK/XLgkngdQfP2/MSJycnJyddee42Zv/rVr/r+TctRLS4uvvrq"
        "q++8807iCa3rynZt74SsSAjgQr6nuVQsz5yZvXz58s9//vPJycmvfe1rmzdv"
        "vsUC9d577/3zP//z+fPnASC/tbF+cwOHAixIABbEALIAreT2IsC9Mzg1nSCA"
        "mJ8kdQeGNSkOTTbf+vfl6HxllqlXKQE+p0Ba0UNQiMEyEACIACIyAIAoaN2S"
        "D5fLc2cXh4eHf/rTnw4PDx86dGj79u3XuCyHhoYuXLhw5MiRU6dOLS4uAkB2"
        "Y6Zn30ZoISkCoGARIScbd3QA88zZ+ampqVdffXV+fn7Hjh07d+7ctm3baiPb"
        "GHPq1KnjJ04cO3IksX0zGzJd27okJ1IGJBBGsYCCLFhpOfNrcDZqOYEOxvZJ"
        "wFoVtnKcaWz4hzA+B+sR63AnuJrtKsDClOgSyJAsAxAD1FHn9g0AMD+4NDU1"
        "9frrr589e/bhhx/WrpsNAiKy1ibbvZ988kni9UeFjVvru/raoVVDQYBAWAAQ"
        "CoIN2Lm70/XdqbOzpmg++OCDY8eOHThwoKWlpbn5anmfyenp8bGxU6dOJU70"
        "uq5s795N1I5QQkARTs4X2AqCMFeacdQ+/ZdLVGPsg4MfkYLIfoHw9utAbIOm"
        "hn+MymfJvSoyNsKUAJ9X6VdOEgTGVpSt7IIRojAAJuIkUBZqUht3d/p1s7MX"
        "5sP5qOqRRESl1OptYCDItAfNGxsbuxswUFBmEUw8igKMiFxkDLB1R2uQ86aG"
        "ZwtjpTiOjx49Wvk0EazdOdIZ3bSlvqN3A7UqKYIAAwAhCoHYFQOABFCS4CPU"
        "SpnbC9zpiwd6O0843u0ZwwgKPvJdKMdPu3ArDoQ225j5QWwH1nHthPV2Za7n"
        "u45rWZQIxCKcVDkXAECpZoaglAECbN3eUlefmZ1aKM2VwrnYlqyIJNKPCt16"
        "7Ta62cZMU3uDbnKAQSLGin29YmYkibchCEFddy7bmJnfsLg8VyjPhdFCbCOu"
        "ij655DW6mbzf2FJf11kPPkhBAEAIAMCKoCBYARZARkEwwJZZ2HOUZLO3v3Kh"
        "0xf37OnrJ6+mVUPJR4GHUfy0vgkH4lJQn/uByACsa6w3AmSculzgCVuOGWKL"
        "oCv+RERhhtV1o2IQkKAj6GoPovmotFSKwtjGNpm2tau9rJdp8CBQyJDE619V"
        "SQhABAQBOXHbAIMYQJ/yfU2N5YZoPioVSnFkrGEAUJoczwmygZd3wSWIQUoV"
        "7iBXirdVDiICAqCAjSELzJz1vGuKtdzCXu0f3LOnr//264ACtqDlOLlQNl9S"
        "dK3zKgzrsrkfWh4g5waOQuunKtDn9noaG5vqsyxMBiQyQmsV3Wv28AUlFEBx"
        "6l0374KquItgpfBtMutfn1SFViApEXrNS4bFABJ4zZ7X5qOGajoyiEgsACBl"
        "Fq5sU4ggEiAAWyFEQREGAmArEjFE7Pp+XUOtWcsioohOX9y3s++k69zeHmAG"
        "guMZDyLzJaKrycdRmMnWvUx05qafLEBKgM8vWlryQUODRJZLlpJwfUQQEBC8"
        "Rl4RWARBxDCYVfRYaROTzPW8Mu9XP5zkmOF1UaWVJwYBrURYKaNbeT9WVCYU"
        "EQSp6PpiK6xjRgJAEUbAInPZMJuG1o62ljvIumQBBDwzuHfPtk/JqUkXInvc"
        "dyA0z2g0ABCbIOP9SOkzt1pmTXHdSMs6DIfe0LWpfcNmtpEtxLDITCDMyfaq"
        "FUkeImJFmBmlqnjI6qkURFCSbeTKH5i8Z9UjSba8kSUuKyRaMz0DijCzFQC2"
        "VkBErIDY5Nww2cUWESVQsFyycRx3dHS1dt1xER5Eder8/rikqAZVhRmIjwfu"
        "m4ZdNq6n/0mrM7fxwK6jxpbrcAVo3bRj66Zjw+dOupFwIZR6DyBxNTICJVK5"
        "YpmCMAgwrtlAS7zyFf/M2gUj8cvI1YmfCarrAMpVjSgZoVVfhii2SoWkwXyi"
        "BQkSiVgBQgJgwFB4KcZQSFNfV2uu5W7ayiOqgYv7dvR9GmRsTRwwx7M+CZcc"
        "PM18y3nRgOuU0xXg8wul1M5dvfVtGyWK7XxMC1ZQwAoICjOseohhkVuNw0qP"
        "sGpKTRJZtHpeT+IXIIljq64PiRQyIK8yP1ZeQk5ItPLOJHyJmQ2DElyKedlw"
        "VG7r3LJ15/a7uwkiAkBnBveH5Vp3bSk66mB/Le90VJgS4HONru2PPbx7h+UY"
        "I+H5CEMRrOysyqpHosQzs9zpeBJdl8jxim2Aax9X1w++OoKrDImr74SEXcAo"
        "UBKeizEUi/LQjq0bevZ/pl8XaWDwQFS+x7+y48QpAT7XcF33kccfa+3qM6Wi"
        "LBuYjYQBV6bhKqwk8dKAqxT9ytxc27hUx0XYrnmI3PSPysOArDwqdogBmQml"
        "YE1U3Ni788AjT3zGCunJCZ4efNhE9+yHNpBpDtZPNNy63eLr6N3zzKGhn8xf"
        "saXQzqHSxA0KpJIecK3Neo13VKpuebxmfGW6vm4cRVa2xa4TQ1zlJmJciXC7"
        "OiwIKIBIczEvmNb8ho5drQ/v7WjNL/D0hyxAwKjsig0BzJA4tUQhsgUBRAdJ"
        "3WIdODN4cPe2467/WaOLOK7L6RezXZcB3JQAn3c8/MRT41em3v/wXSpbnokQ"
        "tNRrQSt2JdQSKx1QWMlamRYBQEa+dt5MNPgbjVsAQLl+nmVYG9CDkuz+8lW3"
        "qRCDAC0ano2gxJ072776xVaWcG7stLAFZhQbWUIRawUIjAUAsIIcAyNYQBS5"
        "hceHGZSiM+cP7tr5kafsZ5H+jPtSXHQ9iUScWrrRpCrQb5Tcmcbnv/KV3XsO"
        "Wo6laHkqxLlYkqg4FECp2Ly4sue1oiAl5qwQr3Z6VscZbjSOAmvHK48VbQlW"
        "P5gFuJr9hQw0b2Q6wrIISUtjBlabEPcIAurMmYNxrOiu4qtZ6nx6qVyuY6aV"
        "ret0BfjcI5Pv/MY3vxGaePD0x1TSMhMTs8k5olCYCQgAeEXWUKoey6SHClyj"
        "G1XHWW48fn22OCLeMIccK3M/QAxq0ciCoQhE095dBw8+sgWiqTVzVHxvGICk"
        "zpx/dOe2o15wZ+uAmGygXoriTHLe6ykpfv1Xz2ts3vj7v/97u/Y+xsRSimXO"
        "qpkQywYJGLiqlCf7tAQVTiQWc/K0+rjT8UoERFLxapXPJ9n7ZRQsMk1GMmvQ"
        "oLFRa3v3bz/7bKaxRa6l3r1bBwTPnn30jjINOHZR/jSymXUpHg9E+cj6/MYX"
        "v/1fPXXoKZ3x46USLgJORnregBWLwlWvEFT+hOSPFadnZYN27fgaACeTIt9k"
        "PHmWHApELDFYcRYsTkdYrBRvFLb5pnymsREYECUJ7r/nbfIEVHvL4B0dlpyo"
        "ruH1uBisdi6lXqD7CSISNHV87Rv/dXtb6+vvHp6+POTGvlirSpayygRKNICV"
        "lVC4FX0oCdpZ/ZMn3p7rNeCV8eviHxAwsQSwOuGwEbXEVLJSEowRmFVWAxKU"
        "wRiDHAMBY0VE2VQ9OfxZC50jMFBbbrCta/pOP7q8eLKlW6bHX3CcEABilaZE"
        "3ldIAtB0pvGxZ393Y9eG1946PHD203Cx4EYZKBsnY9kn65PVQlDZuxVZFcIG"
        "1859Ijcex2tj41Y0GREhgFh0KFhkLIuKkK2Ah35ni97ghhcWebrilSJiYEYl"
        "YpgUswUgAPOZZwHA9voLnZsnGIHoDqsEaVhY6m/pgOkrL2httDEpAe5XdGx/"
        "5Fsbek4e7Tp84tTo0FlZFh37WGTHZfHJBsgOJiGgCAA3z0mUGgYFKtGkFIoO"
        "BSOGsCL6lsBpzfg9DfHjYIuCFxGrC0simgaAgZlQ2XtR0ora6oc2dE+ABbjb"
        "6Xthrr+lExYu/a6zjuThgSMAAGTq848/90LPtl39Hx07MXBhYuS8lK3jBlJm"
        "KYjjEztoHWQHRQMBCN9BkyGpliQyQLGoWDBiCQUjUBbZCmtRed/vaLD7sLQl"
        "No4EU06iV139GsOCAgQkhg19RpcjIrY2XuzsvvLZhWVhvr9hIxQd2M0sAAAd"
        "HklEQVTNvpQA9z1au7c9s3Hrzn2nz546eeri6MjQuXBxyfV8LGl0gDQoB8FB"
        "cZAVigZQN9rnWi1nDGCFDGAs2grHArGgARWjCIBl65HTnPVac3YrxJvA+KyW"
        "lO+grFTbTAoAoVilrNhkc5kQP9P8L6LaGi92dE3dq/u2tNA/XtjS0jKdEmA9"
        "2AYbenZv6Nm9+8rQ0JkzQ8OjF4ZH5yZHzGKktUtKg0OiGQlQoRCQQiYQxNU9"
        "F5OQOEoKOqxUOQeD2kBF7l3SOddtyGKLa7dA1CGCSBHoiICBQVSESVjo1c1c"
        "AwBMwIwMTAIWCO9ia0xEtTdd6Ng4zXzvHEoayK6f6tAPNAGqaGnf0tK+ZV8Y"
        "Tl84MTw6cfnK9Njk1PTU5bhYUOAlm8IrXkQQWBMvjwIoSBYoic8RQSsAwFml"
        "Mx5lPV3vSzvaTrANQgagDFjJThNgALPGwCUiEiMoYKu7BgwCd+ECElDtTRc7"
        "u2udqlnVWYuOWrrN+wwkvqCUAOsNnud17XqiaxdwaX5yfHJyYuitD0+MXjyj"
        "XE8cFGvZGI6Zk5yBJLIfUXsueR6DoEbQjva18jX5LtSRNKNtlSiLAICx4JIk"
        "vdaTzEwEZGCMMImFhmpsEgAIIwpbQKpUtrsLq7e18eKGrprmfiIQCGL4Q9dl"
        "tn+jqXg7ClBKgPUMCho39DaWlivdX5AwXCps2bH38Yf3RmEUR6GJjBUBBN/V"
        "p4fGLp0/TVq7LXXYF7ALtpE5hwDARsgAFLmSCIzVil1S5Q8gcCwrSTdgjXLr"
        "fMviIouuxGYkFnLti4AItueHOrtmWKC2rEjt+C+UIz8mcei7LH91Gw6sHy9o"
        "SoAbYXFq9MgH77535MTSzLhfl4nD2G+o/9ITj+578tnr36xe+dHgqY9dR2Pe"
        "j/sESwIMspQER1ezxwTWRhZJ9V8BFZNlACRERAdNZm9zR3F+4iQlEQucpG7W"
        "Kv0I2J6/1LFhpsaSuogKnW+VwxZAYCYjvnJe4vivybk5B3RqA6xTlObGB/o/"
        "PvzxmaHBU2xMIv0A8IVHHt/z0MM3ng3DMhuDCtGAFASj6jTMVa3mxkFylXEU"
        "U1F7NJLvagCA/OONYOZG+5XCCgdqs+kRsTU/0t42wwx0O38/EYgodL8dmfxq"
        "ncaYjOd81/JfK7pJ/ROTEmDdYWlu7Oynn5w8c2HwwunS4pKbybiBjosl8tzH"
        "Dj7x7Je/TMGNW1REMVfUGxSIBaJqXWdMgv9hlYdfKpJayYavREZbEREktVQu"
        "TVw407x5R5BrwvwXmgCmR04prDUwTgTbmkY6Omdr3DizrNB/MYzz10ZICxjj"
        "6+AlsX99wxpA6KSxQOsI86MXzpw9+enAxZGLA6Wlgpvxg1zWRKZUKDW0dT9z"
        "6OFHn3rOr2++uRjZRN5BBKysDl1OKlKvjo+o7C4nNeGqHQAMs1jtuhMTl3/4"
        "6hvPfWF535NfFhHMf6FNeGr4dC16vAC25Yc7O+Zqkn4LiAq9b8WlJtQ35pIx"
        "GVd9j81fkbt83cvpCrA+dP3Z0f5jx06cOj9yaSAulZxM1stlbRSHhUK2qfmh"
        "/Y8++fi+zp2Pq5vHfllrLVsElMTMNWvUFa4EDl2rqawYthU7gSygFQQqL8wX"
        "Z6bntvetSKFw89MtYKcuDdzC75Lsl7XWjTXk52ry+QiwaHS+acpNt35vbAJF"
        "f8rmr0hPX8OflAD3N3h+vv/0kfeP91+6cNqUyzrIuNk6G4Vx0eZaOw4e3LZz"
        "d++OXQ+72dt05gqWlnhlOhQRLiPFV2PMkW8aMSQVYggAshZq9MU3Oqlm3eZU"
        "bQZmpuZnWgEmhs7eLEFejGrtOd1pRpcWa/L5WKvA/WZczhPePhyehcD+EcBf"
        "kHuVA6JSFeh+xuzIhbfeeeP4JydKc3NuXZ3yg6hYVFpt6N66o2fz9u2berbt"
        "u5nGf+0ciWgMq4qSzxiTsJBFIVOtnrWmjlCSDZOUFWUQMBCC0QgdgqySlMsr"
        "m+avWWSONv2Ph+D/mLh4Fq4LSmLRbZv7g6YxFgKyt7UWhJTobxrTVPvOgkIB"
        "/h7Hf0nOCgdSN+j9i9GTR19+5ZeXh86A0l59LioUvbps3/5Hd/R2b93e29q9"
        "R91JsHvsiLGQ6D9JXRO0yGi3mAN1+QwRoIAD4BC4BC6RC+w67AO7DueUbGn7"
        "1hMDzxVyxSNPHF2rgq8R5Eflg/NN398m/3ns0uCqiDkQUO1b+t36y5YVALgM"
        "WgEAJC5TigEUUKWcHZABJte6L3DceEcBoWyJFJP9HsNfkjMNGlDblAD3JYZP"
        "vvc3P3plemzIrcuAgI3i7fsOfuHhvd19O+6yAqENxVpYyQUDK2BZiFFKWkgJ"
        "KABF4CrUBEqAHAQCJrYgReDB+b8vHJq1XhNZwhXc8Is28+BI0x904d+NXrhQ"
        "1Xzae/tVdpwt+Y75NNaHx1qmy0HgRXt3XlGOTUL3qtUZFSKWezJysc4P8irX"
        "YJuDUgNK0pO4BpWGgOz3WP0l0XRqA9x/QMSJoU/+9ievzoxfcnwvKha11gLY"
        "kMnu3H+wRoXnBj5Q0SwrTn1msQAWRcQBzqLVCpCBiFy0WkApdAA1gAJxiLUI"
        "GJm7+PNNu59bwq7b1llol3Hb+ExnH4yevQBC7dv7vboxY8gl859OOG/051FJ"
        "k1faeeBKJbBipfIKEpCgjTegKiyY4oKBy8QCjBoUNNZRUO9kW6i+CfL15Twq"
        "giQtiNcoXAKIIBR/n+X/SlMi7z8UZsd//srbM2MXSSk/l9+1afPY5fGZiUsf"
        "nzzWXJ99+nd+z3XvptKTrKQ8VnR9A8yWhTWxj6JEiICAScjVQCQOsaNFISoQ"
        "x0kaLcmVs79o2/07BW6/LQeYGeu/1LrZA+dHXm6MmQDg8LjzxuEmVCKC23dP"
        "3sBFKchmA+JqvYcQCAUZCotRYSmeGUGbVMdwTUPOzzR7mTYn10gt9cvtSjEx"
        "Vs3levp90zwB0J4S4H7CJ8cOnz/7CSBkGtq+/ttf2n3w0GD/Rz9+5ZczY5fe"
        "Pnqka3PHjoef+SzHFxEWa00S4IbaY58MIhKRIlHIhOAoUQq1sFLooEUBElCu"
        "ZYbpk//cuu93C9xx2y+6dGnzf/NH/8Prv5pFRAHJeuZHbzUksr2jeQHVDbac"
        "JW4HvJ27RxSBQkQrpflSab40c54YmAVFq4bGbNCmM+11dc3Y6s60YG79iM0D"
        "QYDF2dFTg8OJd//LX3zioS9+jYh2P/HcUqn089dK81cuv3v40+7N2zL5zjvW"
        "rEAwab+RqEAGAUCANYCDAoBErAkUgUIhAS2iUJQwASgSUEAMCILC8/0/adzz"
        "wq05MDi448//9KuvvT7vOhAbAICMHw3MEzogQk1ti0nvmdXSb+P2tXP/rfX8"
        "q9sYJApQMQFzeXapPLM8e6Zi6diMNG0D1dXubHB62kdfuq9l44EoizJ2/tzo"
        "6BCSemjv/ie/9CxRpbbZoSeffmTfPtf3B8/3n/zkxN2ZFonrHQHYWmZjjTFs"
        "HLAawdPiIjgkDokmdpRoxYpEEytlAUAljbwJAMDEMN//kyyN3+y73njj+T//"
        "06+++face21O7op8E1+j+dyR9Neg7xGCJvTKUXT2VPSrXxX+v3/55H6XjQeC"
        "AFPTMwuT403t3Y8c2Ad+I1Qj7/3Gp774zKbe3cvT0+fPDxcXZ+/0yIoQV4J1"
        "pFryn0GBaAdIgVasERy0jhKFVgtr4OTbFVqCNSFrVQ5cbwy8Mfz8f/zfdrzz"
        "3mwlPO720orAbb+u33eFaFphSoDPO0RkabkQlku7tvW0bztwzavNXX2/9eTB"
        "hg0dg8OXrgyfvtODu5ocrSqtHVfEgkUUigOigR0SRZYQCNhRgiBAlSigG976"
        "G64DR48e+o9/suPNt+d0jbO5oJg2FgUpUgJguFAKY6WdDa0tnudd/4a9Tz76"
        "6IGHZi5fnBi/40Rvo+uUo0QEWQSAEZkAFBJAYvUiiKOEqFJPGhUSAJIovKkn"
        "0cQwc/KnzXSpKv3/4d8dfOfd2VrdVIJi2uAm0o/EVG3dh9VeTQ+GJvBgGsFl"
        "8EWkrimfb6q/yRLR8PgTh070n5mYumMCKKV85SYWhQoFwYIBBMZGTBwyiYZT"
        "UXVqiNFPYFmGT73etvt3fvXBN/7Dvzv4znuzSt9e2bAACm8i/QSlCVyYc3ta"
        "urd3NNebvDIe+mHBn5+EyeHlCcYSXO+EJaCkhunaZDRKCXAfwfd919VepqGh"
        "4ab91jf07P7CwYfmlkpcmq9lUyyO4+mRs9NTk8tLiwNDI27gAwMUmYoADBgL"
        "9FWEXt1dkVthYZg88S///n/6X995b97RYmwtqY3EcTPeKM7hdL/qxpZv7don"
        "7OgYBEF0jJZypZZ63brD2X8xe/LswiCv4oCgRNOoAifX5JXCUCBel+LxQLhB"
        "s66XCXxQt5LsfQd2Tc/MstxmeosLcwP9x86dvzQyMTUzfaW0MEdE2vU5ZopB"
        "WJCQ+R5slEaxu/nAR8eOzziapKbCXFakXSmVhGasxqlTug0afmfnAcNEuNLT"
        "TAEIMYECAG16y3ua803vzR2peo3E0PFB89zu3m/So0rjsab3+6eH7r6sXEqA"
        "3yDqmurr6xuVcysxauza3dReAvdWbdmX56ff/sXP3zt+uDgz6zieIuV4gTgo"
        "Gkk7opAUIpFyUOxn4oBYtXHfJ9mmS9bWLnAEqJiIAKocUIgLl9Syke/u2R2v"
        "KulLQKFantKTG+NeBQoAEKWx2L07P3dq9nzCAVsSYO5uzpsgAp88reVqi6iU"
        "APcVNja3jDVesctzt3iP53lwIxP56p2an3/r1Z+9/stXEdDJeJQLKKPRU+gS"
        "OkpcAAdFAys0LuPi3fsHBVXbnrO55gt3Iv0rNq7Iag4YA6dm8Qst3R4H1blb"
        "BMez58+V+mcG9MuLF7//8FMtTpYIALkn3DegxpnLqDCchsDJbGtqwDIyrfF4"
        "pjbAfYa2nQeDsxcaOrNzEx8hWdfvtJirVv2vdD4VsdbyWlQHrbUi0rPvsX+9"
        "/5BhkHDp3NG3S3Fc6RocMRhgTspioYjYFrlr6W/ZeTbXeg4RiZj5NsKWz1Ws"
        "37W2ABGAiC2MKSA+sLFTjFTjqI2OBoqnEJzmXpw5aX/6yYk/e/ypOGnsjbCn"
        "ceOncwMAenpRWrTXvNQQOknrg8rHk23EdSMbD4T7y3XdrT2bTMSEoN0tohqS"
        "wGMiStSC6tME1ZeISClVHbTWmqhswwID9R48pLU2ABbEEBgBi8IIFsTKPZD+"
        "Wt4fBLhHvQhQvpE9TEqpC8vY5zR5cd0qlxdeUIO04ibalJeReG7AVLYdWGAz"
        "bCkMUThJfa2bnzy4vexV9B2XiNAhrcFbV2bAg+L/7dh+wNpYed3oBPfkgALO"
        "9ocPec49KxVelf4ai3j6PjW3/Hn5L/77m73BGCqX7cZ8kxW7+veet0vVxGGv"
        "VcDw5fmrae9u7MZL+Xgu+MOeg49nurlSss5uW977/dbvf6fvTw4daBWxKQHu"
        "v0XAy3bUKP1JIbcaRNbZceCJOs//Ly/9no9N+X9zYeShW7yHlwk0NbdnV//I"
        "ghLbsHp16AhoWp4vVFNiLKum5iyLxHQ1BBoIJBNIptUpNTvaXU+C8aAQgJmZ"
        "6mt8s43DGpUQIadn/5NZN4DPEBTDMd2R9LseZf1/c2Fk/3W/5ZrPm1AAZEOu"
        "1XUyq0sL8VqVKaeoUCjLilIkjJ09rgUzDqVYGQAQEscJjNcGToF1eZ0JxgO7"
        "BX5zmTYFKA5x+bJIrepN7/5DGce/a+nv2DdwB3O/h2j/ZGgkr/jaEs12bXgF"
        "RwAAWahTXs5zs1UKMK75oKdwIY50tVYj4eZG39Ly//7mP5yPhgnZQ0/c7svt"
        "P/3rsf/8g6n/+/2Pxu9hhGlKgM+Z9EcFLlwWUCYuSzQCNXdE7DvwZNYN7tRB"
        "LqA6dg/kmmuVfiRl4/92emYTMpryBGLppr9r0gcZwAcXALRf7wZ1wEnywZo9"
        "XZewJCFeXSLY54zvAoDnAHkUcHajGLISmzgMw9DwetoGSAmweso0y1QeBgWg"
        "ABFNXLblcak5prJ3/6E638eaYx8EVPv2gVxbrT4fYcxlvjU3ezV5X5bGL2y6"
        "GjeqVv2YLGgNAAAZndBY6aybqbdo4NrAHjFrt7Y0+KQBQJSvbaZrPTk9UwLc"
        "1OoFu6yjYVBrFOk4Ck35MnGtd6l376HArcnOZtbt2wdq93giYl32OyPDa5pz"
        "MemR/Plq7vo12cCVZ3xVkVM662T821r8DlBStYvr9Q16wqYEWH/SL2ZJ8aUb"
        "vSRsynE0XqMQEMDW/YcC/zYcEKvatw3kWs/VeP8JIee9ODa859pxNuoOS5Qo"
        "z7umeCJWunasspKZkshQy+tfPFICANhl5BG+edq4ictxYUpqKJiVVL3dtu+J"
        "bO6mURUs1LFjoL7tfI03HxGz7h8Mj++vwUK4wU9rro3i1ESEK9eCiAxItKb7"
        "hs5UWtTLTaJ+0p3g9SX98aXbTvDWhHZpvPajdu58Lpdx+LrZmS119A5kWwZr"
        "U5OICLLuH1y6fKAmI2GVXBJKYhKIs+YkVOSLCBAhVWwDQVCr+tg7ypPmFZfo"
        "A/D7P9AEQLsE0aiIsrfrPoQKLcdhodZmo0yU2fbthpxjV1cyRNXeO5BtH6zR"
        "50MErn5pePShWjuzs4EVTYYFSTMAhGudnoSI4ggDKIQk6lPABUo60GjHx6Ar"
        "okVMVgBOCbBOYYyLvBQXrxDVHL1PIBzGxZp0oQRB34sNWWdlDVEbtpzNdQzW"
        "Yk4gCjMF7h+Ojey70wuDFTdlEv5dguXrLAqvyjAkHRnOKl9QlOvpYIMSKcUF"
        "YQKA1fPCmtMWTAlwf8OhGVsaR0S+QzvP2tgW7qDlutf7Yl3giqj2nnPZtsGa"
        "PSoUuH84PLb/Li4NrU1aCpMLADgbF9aqSajIvWr/IpYBM66rXM8N2pLg1uW4"
        "kKwAkBrB61TzWYiX78DBv1bRJmtsWJyp/ROl5j/rOzhwR9Kfdf5gdHi/ju+2"
        "DHlsCUXlBAzPLl/b99fzvNXmcjEydfk6CTqqjSuvzIVIBAxX+2Ere7Xjn5WU"
        "APe39JcL04yfIRFCgE0UlWoqIjQ/3/LtF/6toaFaPZ4EPr10cfThuzqxVfZG"
        "ZJTLytNT43PO2piOhmrWm6ApAjA3tbYCVZr7WQznCoWkC5is7uAtDqAwA0dX"
        "R1HSukD3nfSXpuleZAGxjaLSLN7SIRhFmW+/8G/ffuci1hYrRwRavj8yeuCe"
        "XCwbszmAk4sT5K+JYNvsbBYJAQAJihOAju5rulqPMaqfLJUXyUFgsSu9WUWk"
        "jvMCBm0sYqsLiEVJCXC/Sf+9u2K2Ubl403Vgaanha8/9L2+/c1HRHUj/2Ni+"
        "e3jJ+Q0mDuH48qk1K8Dy1qzXmjh5Bmbt/roNXfHmlRldDfMJAa0CBKAoLlbV"
        "tsbJ/U3Y5Re6D7bvYInXjfQ8KARAxKg4Qff6cplNXJy5Xrm31vvGV/7nt94a"
        "0gqwhmlSq3sh/dd9kZOVtgx+1H/RW3WGsciTwbNaBedOLu7KNn390PNVV6lV"
        "pZMj4wQEShzHmZmdQ9YAQLaMi+O/V3/ou50Ht2fbr65797+f9AFqkPHrOKwg"
        "GjZcmPGCJrkq/U7vpv/n7Xcuep7U0rYREbV8d+Sezv2VMxHZtMUePVN4d+HD"
        "x+qfRmUBANHqUsOz/u89/mShSTUAV8KfCPRp7+fGhgqVUtjboE/3z6jeKLKG"
        "o1FFaFkYVUQRr6P5Mw2FuDfrQKk0l0SVWetl9OvDF07UUssNAETQVS/dK73/"
        "eqhAHtqIvzx9fphOVlO8mBhAN0iDrNTbInYv51/vHx1SqBKlv2k3tdYX/s/X"
        "/+J8+HbBXyhl5sdy5z/Uh3984QSBm64AKdasA8BxuTDr+K0Z/VpY+EAaasqP"
        "IQLfeWl0eP89WZ9uNpl57fZhUn/7/rHnd5d/K/9oKHRN73lR8QnnXy6cH0Vw"
        "EnNdEAmgfpviaPrw1BSCRgIRKwQoBOsoISYlwD1SsUSsFV1+JbRHoeZIh/rs"
        "SxcH9/3aT47BbbWP5XFiov8/DY5+4dEte+Txq+HSuvyzub9FEaSrdRxRBBCZ"
        "WRQCIIgwCCAi4zUqVqoCpQAEseK46jhJrdKvNezf88K5sW/SvZuDhG7FAVCY"
        "2aLzdcV/fPdEOTdyVQKMn/XrbjipExGiWt8ikhLgXsywVrt03KFjtd50hJ07"
        "vv3qO98TkTjouTccuGVIkwAhabaS6RIw8vGVS86qartP538rCAIUuUFIOBHc"
        "fN9E0n2AdO4XVq467tAx5pqMQiLYsevFX73/4ooRLHHQo+5F0dkbZjRIEv2s"
        "K0KsPWmrc/r7x4JVU7633Pav3O/9dt9vWy7f0MLBm8T/ecpJCfCgz/2aTrjq"
        "aK23m2Dbjj98+4NvXSOjoX9vOHBDNYYctWqdwM3bcbS8/NryYbUqGsqidU3d"
        "TTNgFKobyXpXj0oJ8AA7f1j5zglPHWGGWjITEXHHtj94/8g3bvjqr4MDRCSk"
        "rwnrVz48upNeO37irfn3fQAyPrBLxle3jg4kuIYDlvm3dnw59QLdJ8J6z/0V"
        "Qp57Gu2HK2IFwAAEJMAChJGwz3h1F0wp2br9O+8e+/otDhn6PV75ooW7LTxI"
        "a3QwQsVImKhGspq3oOvhsUf11Pypv4tOt7Q3udqNTVSaLCsMbn180pqNIYVR"
        "bP/odx9uHno6JcCDQQAGESQiAECFYLTnfgzmPUve3EIQxRlUTfVZP9PQWA47"
        "PX9bwaLjvVeXuTB4wXZvHHfd6MBD33rj3d+9rb8/4YAxdxEIjUircrgU8u22"
        "oN08AcB8cR4LAlaYOcmNR0ROCgZTJcuZLFi0YGOkTFNj/sCjemvrU+tA+uEB"
        "DIWoFIJ2AWIWUeQyGUGyLAp8oyyUluKlQrxc4lIpDKM4Co1htlasiWNbqZau"
        "lPIzXYj/Xa4uqG/MtGSzdXV+JuMGQeB5nuM4RBTLjpkldrPUf1GXlqYOn5hu"
        "aJiM4xgAurq6nFVVda8hZ+j3OOYi33UyAAAoFNLViwUBNgaBxQoggiBYCwoF"
        "kYiYCJQCTYq0IKJCJEAREQso+ab6jZuguyNoMVv9hceyw7sxU3YKERTWiWA8"
        "KATQWi8uw8mzF6an58NyGIURA5dKoYgwmygyK9tC4LracRzXdR3P8zzPcz3P"
        "dYJM4Hmevwqe57mum4i71lprXa2ovsZP4kFTEzC3MOeNMWEYFgqFc+fOLSws"
        "lMOSo926+owmcpygznVbOjuTPpax7mlRZyMUU45DcSw5NTiYBJPiDihiREyZ"
        "ks05QUEkrdHV5CrUBKhAITkIiEjgElgGFivWNjTWdW7GLV1Om9rsyrb82BOV"
        "Y1dz4DoX15tgPCAEUEqJ07y4eHZ2fjEpt6Act7ExIK0dx3GUqgi947jXofqS"
        "1loptVrcVzcZuI0rhkhr7XlefX19W1ubMSaKonK5XCwWl5eXZ2ZmLi4uLhw+"
        "vLi8LDZ2XTfptOEFnuu6QeC4ikhrRynHVSCQCTQAGANZg+g5wAAkZn4ZYxRN"
        "oBxyFGgHlCJHASpwkBQCImkQBhbLxmZyQdcW6u1y2v2NCje3XXq2crrj8ODg"
        "AVKBcrlcX19fe3t7siBorR3HIdf1VmR6NZK+GEoppVS1d0YiyvAZYkuTDyZf"
        "4XleNpttamqy1iZ8CMMwDMNyuZz8P/kjDMNiMVyI4zCugI0xhplN0t7mC+7m"
        "93FUBCmTEdCoNSqFSpGDoBAQBa0Y62X8rk168ya90etQ2NM6/FzlnP7/du3Y"
        "CAAQBIJgRAPff4VWYWBq7OCwWwOXAGv0Km/QL1CSJK2aPIFVPTwnzR736+YM"
        "BAACAAGAAEAAIAAQAAgABAACAAGAAEAAIAAQAAgABAACAAGAAEAAIAAQAAgA"
        "BAACAAGAAEAAIAAQAAgABAACAAGAAEAAIAAQAAgABAACAAGAAEAAIAAQAAgA"
        "BAACAAEgABAACAAEAAIAAYAAQAAgABAACAAEAN/b0mZho3Q2iFYAAAAASUVO"
        "RK5CYII=")


def getErrorImage():
    """Generate 'pending' image from embedded data."""

    return base64.b64decode(
    "iVBORw0KGgoAAAANSUhEUgAAAQAAAAEACAYAAABccqhmAAAAAXNSR0IArs4c6QAAAAlwSFlz"
    "AAALEwAACxMBAJqcGAAAAAd0SU1FB9wDCQACMfBoIRYAAAAZdEVYdENvbW1lbnQAQ3JlYXRl"
    "ZCB3aXRoIEdJTVBXgQ4XAAAMd0lEQVR42u3b23Pb5oGG8RcnAiQIgpRdUbJ8jONNk4tM053O"
    "XvZv701vsrvqtJMeMtvYUZSKlHWgxBNAEOBeGGKhCKQkS4oZ6/ldaUSK/AYiHnz4ABrHf/3r"
    "rBfHAnC/NF1Xdi+O9f3JCVsDuG/CUCZbAbi/CABAAAAQAAAEAAABAEAAABAAAAQAAAEAQAAA"
    "EAAABAAAAQBAAAAQAAAEAAABAEAAABAAAAQAAAEAQAAAEAAABAAAAQBAAAAQAAAEAAABAEAA"
    "ABAAAAQAAAEAQAAAEAAABAAAAQAIAJsAIAAACAAAAgCAAAAgAAAIAAACAIAAACAAAAgAAAIA"
    "gAAAIAAACAAAAgCAAAAgAAAIAAACAIAAACAAAAgAAAIAgAAAIAAACAAAAgCAAAAgAAAIAAAC"
    "AIAAACAAAAgAAAIAgAAABAAAAQBAAAAQAAAEAAABAEAAABAAAAQAAAEAQAAAEAAABAAAAQCw"
    "2mw2wWr4/dOn85//sLNz7ccBAvABd9r3xc4MAgDcwawJBOBOLfvQXffDyQcYHwKLgAABAMAp"
    "AH7x57OmYehxEKheqch3HLmWpWGSaDCZqDMcqj+Z3Po412s1rVWranmestlMg8lEPw4G6kXR"
    "pa9XtW1tBYFqjiPfcSRJwyTRMEn0Y7+vaDpdOoZlv+O0igDcK03P02dra/Ls8//Whuuq4bra"
    "qNf1utfTbr9/a+/52dqaNur1c7/zbFsPqlV9d8l7bdXretFsyjLPT0QrlqWW52nT9/Vdr6d/"
    "DQb8cwkAlml5nr5cX5ckzWYzvTk50cF4rHg6VcN19arVUtVx9LLVkqRbicAnzaZanqdv3r7V"
    "SRzLNAxt1Ot6HoYyDEOfNJt6OxopTtOLO38Q6NN8LNMs0/8dH+s4imTkIfu01ZJtmnq1tiZJ"
    "5yJwdmTnKgBrAJBkSHqV70yS9M3BgXZOTzVKEqWzmY6jSNvdrqZZJkl6HoaqWNaN33fD9/U/"
    "nY4OxmMlWaY4TfX9yYk6+c5qGIaeNBoX/s4xTT0PQ0lSNptpu9tVdzjUJE0Vp6m6w6H+1O0q"
    "m83ejbfZlG3yUSUAKJ9KB4Gq+flzP451OB5feE6SZdofDiVJlmnqacmOeV0/9PtK8qgUdfL3"
    "kaR6pXLh8cdBMN+hO8OhRkly4TnDJFE3fx3HNPU4CPhHEwCUabju/Oed09OFz9srTKPLdszr"
    "OioJjSSNCzu0VzLT8AvvvV+IxU91LwkJWAOA3q2kn/ni4cPFpwqG8e+dMJ8x3ETZCr0kTQqz"
    "grJTjVphvIte46ePVW0+qgQAlwaguJMv/cffwjl1mp+jL12fKBlPMQqTkgXCssfcW1izAAH4"
    "KMVpqlq+Q/9xd7f0vHxVx1uxrNKrBD8NRbwkFGAN4F4rLqLdxtT+ro0LU3tvydS++Nh4yakC"
    "CMC9Vry7L/S8lR/vsDDe9Vpt4fPavj//eVByB2NamOkYfAwIwH31Y7+vOD9CPgmCpUfVVbDb"
    "78/vSWj7fukCX81x5gFIsqz0xqXiIqHLIiEBuK/S2UzfHh1JeneN/7fttp42GvIdR5ZhyDQM"
    "VW1bTc/TJ82mfre5+UHHm2SZ3pyczMf71caG1ms1VSxLFctS2/f1VbstM19AfNPrzYNRNCic"
    "+jxrNFgofA9k8yNxFEX6y/6+XuXfBXjRbOpFs7nSsxZJehGGckxTn5dcvkyzbOl3AV73enpY"
    "rcoyTW3U6xe+k8DtwQTg3kXg6709bdXrClxXNdtW1XGUZpkmaarxdKrjKNLRFb6l93NF4Gg8"
    "1lYQyHcc1fIFzFH+bcDdBd8GPBOnqb7e29PjRkOh68q1LDmmeeVLoZCM19vbs+/z6RiA++NZ"
    "GLIGANxnBAAgAAAIAAACAIAAACAAAAgAAAIAgAAAIAAACAAAAgCAAAAgAAAIAAACAIAAACAA"
    "AAgAAAIAgAAAIAAACAAAAgCAAAAgAAAIAAACAIAAACAAAK7LZhN8/H7/9On85z/s7Fz78Z9j"
    "DD/364AZAEAA2AQApwD4iDFVBjMAAAQAAKcAl/JsWy3PU8vzVLVtuZYl0zAUp6mi6VQncazd"
    "fl/ZbHbhb/9zY0P1SkWS9O3RkfYGg4Xvs1mv6z/W1iRJ/clE/9vp3No4ztx05fw2xnDhyGMY"
    "Wq/V1PI8NT1PaZapP5lofzTS4Xh88yObYehxEKheqch3HLmWpWGSaDCZqDMcqj+Z8CEnAIv9"
    "16NHpb+vmaZqjqO1alVbQaB/HB7qKIrOPWdvMNCrfKfe9P1LA3CmU/K8m4xjFbbFIr9+8EC/"
    "qtX+/QvLUtVxtO77+uH0VN/1eu893qbn6bO1NXn2+Y93w3XVcF1t1Ot63etpt98nAOzq5YZJ"
    "ooPRSMdRpDhNNUlTebat0HUVuq7WazVVLEu/fvBAX+/tKcmy+d/uj0Z62WrJNAwFrivfcTRM"
    "kgvv4TuOgnymkM1m2h+NbnUcq7AtyrxsNtX0PP398FDHeTBanqdPWy3ZpqknjYay2UxvTk6u"
    "PdaW5+nL9XVJ0ix/jYPxWPF0qobr6lWrparj6GWrJUn3PgKsASzw33t7enNyopM4VjSdKpvN"
    "NEoS7Q0G+vvhof6ZH6Ecy9KzMDz3t9Ms00FhZ97w/dL32Cgc/Q9GI01LdpybjGMVtsWiWc92"
    "p6PucKhJHpTucKjtbnd+GvE4COSY1/t4GpJe5Tu2JH1zcKCd01ONkkTpbKbjKNJ2tzvfzs/D"
    "UBXLIgC4vu5wOP/57Hz/3GlA4fG278so+bC2C1PgZacJNxnHKmyLsuePp9MLvx8liTr5a1mm"
    "qceNxrXGsRUEqjrOu/WUOC5dS0iyTPuF93h6zffgFOA+bRzT1Fa9rtDz5FqWXMuSVXJUqtkX"
    "N2MvihRNp/JsW45l6WGtpreFWcHDWk1OfvSJplP14vhOxrEK2+JCAEpOdeanT8OhHuUzo3q+"
    "M19Vw3XnP++cni583t5goEdB8EGjSQBW3Ga9rpfNZumH/KecBdPIzmCg583m/DTg7YLTgs4l"
    "i4Q3HccqbIuiuOTofyYqPFa9ZtCKz//i4cPFpwqGcW4dhgDgnDBfLDIMQ9lspt1+XwejkeI0"
    "VZKmOrvYVby8VhqA4VDPwlCGYaiVHznjNJVrWWp53nyhqlOYQt/FOFZhWxRN0vRKj133/LwY"
    "gOJOftnMhgDgwrnk2Qfon8fH+lfJEdq9ypEuTXUcRVqrVmUYhtq+r53TU234/vz1z1bW73Ic"
    "q7Atiip5CBc9dpVQLNretXyH/uPu7p1cEfnYsAhYIiicFx4suCnlqtPT4tH9bNW/Xbz2v+Do"
    "f9vjWIVtMQ/GkucXr92Pl5wqlBkVLrXe96k9AbjJtKgwLbQWTCWLl/CWORiNlORHsqpt63kY"
    "zneYJE3PXS68y3GswrY4s168AWjJY4OSeyeWKd7dF+anWCAA11Y8krRLruE3KpWlH+Kimc6v"
    "ehcvO3VHI81+pnGswraYB8P3S2cNNduexyTNMu0uWckv82O/P19gfBIEF+4EBAG4kuIdeVtB"
    "MD9qe5alzXpdX66vX+sOsuIqf3FxqnPJtf/bHscqbAvp3T0SX7Xb8zsIK5al9VpNv9nYkJlv"
    "n91+/9rn8Olspm+Pjt7NVkxTv2239bTRkO84sgxDpmGoattqep4+aTb1u81NZrvs7uVHkgfV"
    "qlqeJ9s09SwMz93h1hkM9F2vpydXvIlkmCTqTybnzqf7cVx6e/BdjmMVtoX0bjHRsyx9vuBS"
    "3Q+np+91G7AkHUWR/rK/r1f5dwFeNJt6kV+KBQG4sj/v7+tRva5131fNcTRNU53EsXpxfO7O"
    "tysf9QYDBfkXhM6Ogh9iHKuwLSTpb4eHakeRWp6n0HVv9duAR1Gkr/f2tFWvK3Bd1WxbVcdR"
    "mmWapKnG06mOo+jOvjz1S2K83t6eff+etQXwy/UsDFkDAO4zAgAQAAAEAAABAEAAABAAAAQA"
    "AAEAQAAAEAAABAAAAQBAAAAQAAAEAAABAEAAABAAAAQAAAEAQAAAEAAABAAAAQBAAAAQAAAE"
    "AAABAEAAABAAAAQAAAEAQAAAEAAABAAAAQBAAAACwCYACAAAAgCAAAAgAAAIAAACAIAAACAA"
    "AAgAAAIAgAAAIAAACAAAAgCAAAAgAAAIAAACAIAAACAAAAgAAAIAgAAAIAAACAAAAgCAAAAg"
    "AAAIAAACAIAAACAAAAgAAAIAgAAAIAAAAQBAAAAQAAAEAAABAEAAABAAAAQAAAEAQAAAEAAA"
    "BAAAAQBAAACsNrvpulIYsiWAe6bpuvp/KMHzybJ4Z08AAAAASUVORK5CYII=")
